searchNodes=[{"doc":"Helpers for working with HTML strings and templates. When used, it imports the given modules: Phoenix.HTML - functions to handle HTML safety; Phoenix.HTML.Tag - functions for generating HTML tags; Phoenix.HTML.Form - functions for working with forms; Phoenix.HTML.Link - functions for generating links and urls; Phoenix.HTML.Format - functions for formatting text; HTML Safe One of the main responsibilities of this module is to provide convenience functions for escaping and marking HTML code as safe. By default, data output in templates is not considered safe: &lt; % = &quot;&lt;hello&gt;&quot; % &gt; will be shown as: &amp; lt ; hello &amp; gt ; User data or data coming from the database is almost never considered safe. However, in some cases, you may want to tag it as safe and show its &quot;raw&quot; contents: &lt; % = raw &quot;&lt;hello&gt;&quot; % &gt; Keep in mind most helpers will automatically escape your data and return safe content: &lt; % = content_tag :p , &quot;&lt;hello&gt;&quot; % &gt; will properly output: &lt; p &gt; &amp; lt ; hello &amp; gt ; &lt; / p &gt;","ref":"Phoenix.HTML.html","title":"Phoenix.HTML","type":"module"},{"doc":"Escapes the HTML entities in the given term, returning iodata. iex&gt; html_escape ( &quot;&lt;hello&gt;&quot; ) { :safe , [ [ [ ] | &quot;&amp;lt;&quot; ] , &quot;hello&quot; | &quot;&amp;gt;&quot; ] } iex&gt; html_escape ( &#39;&lt;hello&gt;&#39; ) { :safe , [ &quot;&amp;lt;&quot; , 104 , 101 , 108 , 108 , 111 , &quot;&amp;gt;&quot; ] } iex&gt; html_escape ( 1 ) { :safe , &quot;1&quot; } iex&gt; html_escape ( { :safe , &quot;&lt;hello&gt;&quot; } ) { :safe , &quot;&lt;hello&gt;&quot; }","ref":"Phoenix.HTML.html#html_escape/1","title":"Phoenix.HTML.html_escape/1","type":"function"},{"doc":"Escapes HTML content to be inserted a JavaScript string. This function is useful in JavaScript responses when there is a need to escape HTML rendered from other templates, like in the following: $ ( &quot;#container&quot; ) . append ( &quot;&lt;%= javascript_escape(render(&quot; post . html &quot;, post: @post)) %&gt;&quot; ) ; It escapes quotes (double and single), double backslashes and others.","ref":"Phoenix.HTML.html#javascript_escape/1","title":"Phoenix.HTML.javascript_escape/1","type":"function"},{"doc":"Marks the given content as raw. This means any HTML code inside the given string won't be escaped. iex&gt; raw ( &quot;&lt;hello&gt;&quot; ) { :safe , &quot;&lt;hello&gt;&quot; } iex&gt; raw ( { :safe , &quot;&lt;hello&gt;&quot; } ) { :safe , &quot;&lt;hello&gt;&quot; } iex&gt; raw ( nil ) { :safe , &quot;&quot; }","ref":"Phoenix.HTML.html#raw/1","title":"Phoenix.HTML.raw/1","type":"function"},{"doc":"Converts a safe result into a string. Fails if the result is not safe. In such cases, you can invoke html_escape/1 or raw/1 accordingly before.","ref":"Phoenix.HTML.html#safe_to_string/1","title":"Phoenix.HTML.safe_to_string/1","type":"function"},{"doc":"Provides ~E sigil with HTML safe EEx syntax inside source files. Does not raise on attempts to interpolate with \#{} , but rather shows those characters literally, so it should be preferred over ~e . iex&gt; ~E&quot;&quot;&quot; ...&gt; Hello &lt;%= &quot;world&quot; %&gt; ...&gt; &quot;&quot;&quot; { :safe , [ &quot;Hello &quot; , &quot;world&quot; , &quot; \\n &quot; ] }","ref":"Phoenix.HTML.html#sigil_E/2","title":"Phoenix.HTML.sigil_E/2","type":"macro"},{"doc":"Provides ~e sigil with HTML safe EEx syntax inside source files. Raises on attempts to interpolate with \#{} , so ~E should be preferred. iex&gt; ~e&quot;&quot;&quot; ...&gt; Hello &lt;%= &quot;world&quot; %&gt; ...&gt; &quot;&quot;&quot; { :safe , [ &quot;Hello &quot; , &quot;world&quot; , &quot; \\n &quot; ] }","ref":"Phoenix.HTML.html#sigil_e/2","title":"Phoenix.HTML.sigil_e/2","type":"macro"},{"doc":"Guaranteed to be safe","ref":"Phoenix.HTML.html#t:safe/0","title":"Phoenix.HTML.safe/0","type":"type"},{"doc":"May be safe or unsafe (i.e. it needs to be converted)","ref":"Phoenix.HTML.html#t:unsafe/0","title":"Phoenix.HTML.unsafe/0","type":"type"},{"doc":"This is an implementation of EEx.Engine that guarantees templates are HTML Safe. The encode_to_iodata!/1 function converts the rendered template result into iodata.","ref":"Phoenix.HTML.Engine.html","title":"Phoenix.HTML.Engine","type":"module"},{"doc":"Encodes the HTML templates to iodata.","ref":"Phoenix.HTML.Engine.html#encode_to_iodata!/1","title":"Phoenix.HTML.Engine.encode_to_iodata!/1","type":"function"},{"doc":"Helpers related to producing HTML forms. The functions in this module can be used in three distinct scenarios: with changeset data - when information to populate the form comes from a changeset with limited data - when a form is created without an underlying data layer. In this scenario, you can use the connection information (aka Plug.Conn.params) or pass the form values by hand outside of a form - when the functions are used directly, outside of form_for We will explore all three scenarios below. With changeset data The entry point for defining forms in Phoenix is with the form_for/4 function. For this example, we will use Ecto.Changeset , which integrates nicely with Phoenix forms via the phoenix_ecto package. Imagine you have the following action in your controller: def new ( conn , _params ) do changeset = User . changeset ( % User { } ) render conn , &quot;new.html&quot; , changeset : changeset end where User.changeset/2 is defined as follows: def changeset ( user , params \\\\ %{ } ) do cast ( user , params , [ :name , :age ] ) end Now a @changeset assign is available in views which we can pass to the form: &lt; % = form_for @changeset , Routes . user_path ( @conn , :create ) , fn f -&gt; % &gt; &lt; label &gt; Name : &lt; % = text_input f , :name % &gt; &lt; / label &gt; &lt; label &gt; Age : &lt; % = select f , :age , 18 .. 100 % &gt; &lt; / label &gt; &lt; % = submit &quot;Submit&quot; % &gt; &lt; % end % &gt; form_for/4 receives the Ecto.Changeset and converts it to a form, which is passed to the function as the argument f . All the remaining functions in this module receive the form and automatically generate the input fields, often by extracting information from the given changeset. For example, if the user had a default value for age set, it will automatically show up as selected in the form. A note on :errors If no action has been applied to the changeset or action was set to :ignore , no errors are shown on the form object even if the changeset has a non-empty :errors value. This is useful for things like validation hints on form fields, e.g. an empty changeset for a new form. That changeset isn't valid, but we don't want to show errors until an actual user action has been performed. Ecto automatically applies the action for you when you call Repo.insert/update/delete, but if you want to show errors manually you can also set the action yourself, either directly on the Ecto.Changeset struct field or by using Ecto.Changeset.apply_action/2 . With limited data form_for/4 expects as first argument any data structure that implements the Phoenix.HTML.FormData protocol. By default, Phoenix implements this protocol for Plug.Conn and Atom . This is useful when you are creating forms that are not backed by any kind of data layer. Let's assume that we're submitting a form to the :new action in the FooController : &lt; % = form_for @conn , Routes . foo_path ( @conn , :new ) , [ as : :foo ] , fn f -&gt; % &gt; &lt; % = text_input f , :for % &gt; &lt; % = submit &quot;Search&quot; % &gt; &lt; % end % &gt; form_for/4 uses the Plug.Conn to set input values from the request parameters. Alternatively, if you don't have a connection, you can pass :foo as the form data source and explicitly pass the value for every input: &lt; % = form_for :foo , Routes . foo_path ( MyApp.Endpoint , :new ) , fn f -&gt; % &gt; &lt; % = text_input f , :for , value : &quot;current value&quot; % &gt; &lt; % = submit &quot;Search&quot; % &gt; &lt; % end % &gt; Without form data Sometimes we may want to generate a text_input/3 or any other tag outside of a form. The functions in this module also support such usage by simply passing an atom as first argument instead of the form. &lt; % = text_input :user , :name , value : &quot;This is a prepopulated value&quot; % &gt; Nested inputs If your data layer supports embedding or nested associations, you can use inputs_for to attach nested data to the form. Imagine the following Ecto schemas: defmodule User do use Ecto.Schema schema &quot;users&quot; do field :name embeds_one :permalink , Permalink end end defmodule Permalink do use Ecto.Schema embedded_schema do field :url end end In the form, you now can: &lt; % = form_for @changeset , Routes . user_path ( @conn , :create ) , fn f -&gt; % &gt; &lt; % = text_input f , :name % &gt; &lt; % = inputs_for f , :permalink , fn fp -&gt; % &gt; &lt; % = text_input fp , :url % &gt; &lt; % end % &gt; &lt; % end % &gt; The default option can be given to populate the fields if none is given: &lt; % = inputs_for f , :permalink , [ default : % Permalink { title : &quot;default&quot; } ] , fn fp -&gt; % &gt; &lt; % = text_input fp , :url % &gt; &lt; % end % &gt; inputs_for/4 can be used to work with single entities or collections. When working with collections, :prepend and :append can be used to add entries to the collection stored in the changeset. CSRF protection The form generates a CSRF token by default. Your application should check this token on the server to avoid attackers from making requests on your server on behalf of other users. Phoenix by default checks this token. When posting a form with a host in its address, such as &quot;//host.com/path&quot; instead of only &quot;/path&quot;, Phoenix will include the host signature in the token and validate the token only if the accessed host is the same as the host in the token. This is to avoid tokens from leaking to third party applications. If this behaviour is problematic, you can generate a non-host specific token with Plug.CSRFProtection.get_csrf_token/0 and pass it to the form generator via the :csrf_token option. Phoenix.LiveView integration Because Phoenix.LiveView is unable to compute diffs inside anonymous functions, Phoenix.HTML provides form_for/3 that works without passing an anonymous function. Inside live views, instead of &lt; % = form_for @changeset , url , opts , fn f -&gt; % &gt; &lt; % = text_input f , :name % &gt; &lt; % end % &gt; you would write &lt; % = f = form_for @changeset , url , opts % &gt; &lt; % = text_input f , :name % &gt; &lt; / form &gt; In the second case, the form_for emits only the opening of the &lt;form&gt; tag, which then needs to be closed explicitly in HTML. Since the anonymous function has been removed, Phoenix.LiveView is able to optimize forms too. Although outside of live views, we recommend using the first construct.","ref":"Phoenix.HTML.Form.html","title":"Phoenix.HTML.Form","type":"module"},{"doc":"Defines the Phoenix.HTML.Form struct. Its fields are: :source - the data structure given to form_for/4 that implements the form data protocol :impl - the module with the form data protocol implementation. This is used to avoid multiple protocol dispatches. :id - the id to be used when generating input fields :index - the index of the struct in the form :name - the name to be used when generating input fields :data - the field used to store lookup data :params - the parameters associated to this form in case they were sent as part of a previous request :hidden - a keyword list of fields that are required for submitting the form behind the scenes as hidden inputs :options - a copy of the options given when creating the form via form_for/4 without any form data specific key :action - the action the form is meant to submit to :errors - a keyword list of errors that associated with the form","ref":"Phoenix.HTML.Form.html#__struct__/0","title":"Phoenix.HTML.Form.__struct__/0","type":"function"},{"doc":"Generates a checkbox. This function is useful for sending boolean values to the server. Examples # Assuming form contains a User schema checkbox ( form , :famous ) #=&gt; &lt;input name=&quot;user[famous]&quot; type=&quot;hidden&quot; value=&quot;false&quot;&gt; #=&gt; &lt;input checked=&quot;checked&quot; id=&quot;user_famous&quot; name=&quot;user[famous]&quot; type=&quot;checkbox&quot; value=&quot;true&quot;&gt; Options :checked_value - the value to be sent when the checkbox is checked. Defaults to &quot;true&quot; :hidden_input - controls if this function will generate a hidden input to submit the unchecked value or not. Defaults to &quot;true&quot; :unchecked_value - the value to be sent when the checkbox is unchecked, Defaults to &quot;false&quot; :value - the value used to check if a checkbox is checked or unchecked. The default value is extracted from the form data if available All other options are forwarded to the underlying HTML tag. Hidden fields Because an unchecked checkbox is not sent to the server, Phoenix automatically generates a hidden field with the unchecked_value before the checkbox field to ensure the unchecked_value is sent when the checkbox is not marked. Set hidden_input to false If you don't want to send values from unchecked checkbox to the server.","ref":"Phoenix.HTML.Form.html#checkbox/3","title":"Phoenix.HTML.Form.checkbox/3","type":"function"},{"doc":"Generates a color input. Warning: this feature isn't available in all browsers. Check http://caniuse.com/#feat=input-color for further informations. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#color_input/3","title":"Phoenix.HTML.Form.color_input/3","type":"function"},{"doc":"Generates a date input. Warning: this feature isn't available in all browsers. Check http://caniuse.com/#feat=input-datetime for further informations. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#date_input/3","title":"Phoenix.HTML.Form.date_input/3","type":"function"},{"doc":"Generates select tags for date. Check datetime_select/3 for more information on options and supported values.","ref":"Phoenix.HTML.Form.html#date_select/3","title":"Phoenix.HTML.Form.date_select/3","type":"function"},{"doc":"Generates a datetime-local input. Warning: this feature isn't available in all browsers. Check http://caniuse.com/#feat=input-datetime for further informations. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#datetime_local_input/3","title":"Phoenix.HTML.Form.datetime_local_input/3","type":"function"},{"doc":"Generates select tags for datetime. Examples # Assuming form contains a User schema datetime_select form , :born_at #=&gt; &lt;select id=&quot;user_born_at_year&quot; name=&quot;user[born_at][year]&quot;&gt;...&lt;/select&gt; / #=&gt; &lt;select id=&quot;user_born_at_month&quot; name=&quot;user[born_at][month]&quot;&gt;...&lt;/select&gt; / #=&gt; &lt;select id=&quot;user_born_at_day&quot; name=&quot;user[born_at][day]&quot;&gt;...&lt;/select&gt; — #=&gt; &lt;select id=&quot;user_born_at_hour&quot; name=&quot;user[born_at][hour]&quot;&gt;...&lt;/select&gt; : #=&gt; &lt;select id=&quot;user_born_at_min&quot; name=&quot;user[born_at][minute]&quot;&gt;...&lt;/select&gt; If you want to include the seconds field (hidden by default), pass second: [] : # Assuming form contains a User schema datetime_select form , :born_at , second : [ ] If you want to configure the years range: # Assuming form contains a User schema datetime_select form , :born_at , year : [ options : 1900 .. 2100 ] You are also able to configure :month , :day , :hour , :minute and :second . All options given to those keys will be forwarded to the underlying select. See select/4 for more information. For example, if you are using Phoenix with Gettext and you want to localize the list of months, you can pass :options to the :month key: # Assuming form contains a User schema datetime_select form , :born_at , month : [ options : [ { gettext ( &quot;January&quot; ) , &quot;1&quot; } , { gettext ( &quot;February&quot; ) , &quot;2&quot; } , { gettext ( &quot;March&quot; ) , &quot;3&quot; } , { gettext ( &quot;April&quot; ) , &quot;4&quot; } , { gettext ( &quot;May&quot; ) , &quot;5&quot; } , { gettext ( &quot;June&quot; ) , &quot;6&quot; } , { gettext ( &quot;July&quot; ) , &quot;7&quot; } , { gettext ( &quot;August&quot; ) , &quot;8&quot; } , { gettext ( &quot;September&quot; ) , &quot;9&quot; } , { gettext ( &quot;October&quot; ) , &quot;10&quot; } , { gettext ( &quot;November&quot; ) , &quot;11&quot; } , { gettext ( &quot;December&quot; ) , &quot;12&quot; } , ] ] You may even provide your own localized_datetime_select/3 built on top of datetime_select/3 : defp localized_datetime_select ( form , field , opts \\\\ [ ] ) do opts = Keyword . put ( opts , :month , options : [ { gettext ( &quot;January&quot; ) , &quot;1&quot; } , { gettext ( &quot;February&quot; ) , &quot;2&quot; } , { gettext ( &quot;March&quot; ) , &quot;3&quot; } , { gettext ( &quot;April&quot; ) , &quot;4&quot; } , { gettext ( &quot;May&quot; ) , &quot;5&quot; } , { gettext ( &quot;June&quot; ) , &quot;6&quot; } , { gettext ( &quot;July&quot; ) , &quot;7&quot; } , { gettext ( &quot;August&quot; ) , &quot;8&quot; } , { gettext ( &quot;September&quot; ) , &quot;9&quot; } , { gettext ( &quot;October&quot; ) , &quot;10&quot; } , { gettext ( &quot;November&quot; ) , &quot;11&quot; } , { gettext ( &quot;December&quot; ) , &quot;12&quot; } , ] ) datetime_select ( form , field , opts ) end Options :value - the value used to select a given option. The default value is extracted from the form data if available :default - the default value to use when none was given in :value and none is available in the form data :year , :month , :day , :hour , :minute , :second - options passed to the underlying select. See select/4 for more information. The available values can be given in :options . :builder - specify how the select can be build. It must be a function that receives a builder that should be invoked with the select name and a set of options. See builder below for more information. Builder The generated datetime_select can be customized at will by providing a builder option. Here is an example from EEx: &lt; % = datetime_select form , :born_at , builder : fn b -&gt; % &gt; Date : &lt; % = b . ( :day , [ ] ) % &gt; / &lt; % = b . ( :month , [ ] ) % &gt; / &lt; % = b . ( :year , [ ] ) % &gt; Time : &lt; % = b . ( :hour , [ ] ) % &gt; : &lt; % = b . ( :minute , [ ] ) % &gt; &lt; % end % &gt; Although we have passed empty lists as options (they are required), you could pass any option there and it would be given to the underlying select input. In practice, we recommend you to create your own helper with your default builder: def my_datetime_select ( form , field , opts \\\\ [ ] ) do builder = fn b -&gt; ~e&quot;&quot;&quot; Date: &lt;%= b.(:day, []) %&gt; / &lt;%= b.(:month, []) %&gt; / &lt;%= b.(:year, []) %&gt; Time: &lt;%= b.(:hour, []) %&gt; : &lt;%= b.(:minute, []) %&gt; &quot;&quot;&quot; end datetime_select ( form , field , [ builder : builder ] ++ opts ) end Then you are able to use your own datetime_select throughout your whole application. Supported date values The following values are supported as date: a map containing the year , month and day keys (either as strings or atoms) a tuple with three elements: {year, month, day} a string in ISO 8601 format nil Supported time values The following values are supported as time: a map containing the hour and minute keys and an optional second key (either as strings or atoms) a tuple with three elements: {hour, min, sec} a tuple with four elements: {hour, min, sec, usec} nil","ref":"Phoenix.HTML.Form.html#datetime_select/3","title":"Phoenix.HTML.Form.datetime_select/3","type":"function"},{"doc":"Generates an email input. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#email_input/3","title":"Phoenix.HTML.Form.email_input/3","type":"function"},{"doc":"Generates a file input. It requires the given form to be configured with multipart: true when invoking form_for/4 , otherwise it fails with ArgumentError . See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#file_input/3","title":"Phoenix.HTML.Form.file_input/3","type":"function"},{"doc":"Generates a form tag with a form builder without an anonymous function. This functionality exists mostly for integration with Phoenix.LiveView that replaces the anonymous function for explicit closing of the &lt;form&gt; tag: &lt; % = f = form_for @changeset , Routes . user_path ( @conn , :create ) , opts % &gt; Name : &lt; % = text_input f , :name % &gt; &lt; / form &gt; See the Phoenix.LiveView integration section in module documentation for examples of using this function. See form_for/4 for the available options.","ref":"Phoenix.HTML.Form.html#form_for/3","title":"Phoenix.HTML.Form.form_for/3","type":"function"},{"doc":"Generates a form tag with a form builder and an anonymous function. &lt; % = form_for @changeset , Routes . user_path ( @conn , :create ) , fn f -&gt; % &gt; Name : &lt; % = text_input f , :name % &gt; &lt; % end % &gt; See the module documentation for examples of using this function. Options :as - the server side parameter in which all params for this form will be collected (i.e. as: :user_params would mean all fields for this form will be accessed as conn.params.user_params server side). Automatically inflected when a changeset is given. :method - the HTTP method. If the method is not &quot;get&quot; nor &quot;post&quot;, an input tag with name _method is generated along-side the form tag. Defaults to &quot;post&quot;. :multipart - when true, sets enctype to &quot;multipart/form-data&quot;. Required when uploading files :csrf_token - for &quot;post&quot; requests, the form tag will automatically include an input tag with name _csrf_token . When set to false, this is disabled :errors - use this to manually pass a keyword list of errors to the form (for example from conn.assigns[:errors] ). This option is only used when a connection is used as the form source and it will make the errors available under f.errors :id - the ID of the form attribute. If an ID is given, all form inputs will also be prefixed by the given ID Other options will be passed as html attributes, such as class: &quot;foo&quot; See Phoenix.HTML.Tag.form_tag/2 for more information on the options above.","ref":"Phoenix.HTML.Form.html#form_for/4","title":"Phoenix.HTML.Form.form_for/4","type":"function"},{"doc":"Generates a hidden input. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#hidden_input/3","title":"Phoenix.HTML.Form.hidden_input/3","type":"function"},{"doc":"Generates hidden inputs for the given form.","ref":"Phoenix.HTML.Form.html#hidden_inputs_for/1","title":"Phoenix.HTML.Form.hidden_inputs_for/1","type":"function"},{"doc":"Converts an attribute/form field into its humanize version. iex&gt; humanize ( :username ) &quot;Username&quot; iex&gt; humanize ( :created_at ) &quot;Created at&quot; iex&gt; humanize ( &quot;user_id&quot; ) &quot;User&quot;","ref":"Phoenix.HTML.Form.html#humanize/1","title":"Phoenix.HTML.Form.humanize/1","type":"function"},{"doc":"Returns an id of a corresponding form field. The form should either be a Phoenix.HTML.Form emitted by form_for or an atom.","ref":"Phoenix.HTML.Form.html#input_id/2","title":"Phoenix.HTML.Form.input_id/2","type":"function"},{"doc":"Returns an id of a corresponding form field and value attached to it. Useful for radio buttons and inputs like multiselect checkboxes.","ref":"Phoenix.HTML.Form.html#input_id/3","title":"Phoenix.HTML.Form.input_id/3","type":"function"},{"doc":"Returns a name of a corresponding form field. The first argument should either be a Phoenix.HTML.Form emitted by form_for or an atom. Examples iex&gt; Phoenix.HTML.Form . input_name ( :user , :first_name ) &quot;user[first_name]&quot;","ref":"Phoenix.HTML.Form.html#input_name/2","title":"Phoenix.HTML.Form.input_name/2","type":"function"},{"doc":"Gets the input type for a given field. If the underlying input type is a :text_field , a mapping could be given to further inflect the input type based solely on the field name. The default mapping is: %{ &quot;url&quot; =&gt; :url_input , &quot;email&quot; =&gt; :email_input , &quot;search&quot; =&gt; :search_input , &quot;password&quot; =&gt; :password_input }","ref":"Phoenix.HTML.Form.html#input_type/3","title":"Phoenix.HTML.Form.input_type/3","type":"function"},{"doc":"Returns the HTML5 validations that would apply to the given field.","ref":"Phoenix.HTML.Form.html#input_validations/2","title":"Phoenix.HTML.Form.input_validations/2","type":"function"},{"doc":"Returns a value of a corresponding form field. The form should either be a Phoenix.HTML.Form emitted by form_for or an atom. When a form is given, it will lookup for changes and then fallback to parameters and finally fallback to the default struct/map value. Since the function looks up parameter values too, there is no guarantee that the value will have a certain type. For example, a boolean field will be sent as &quot;false&quot; as a parameter, and this function will return it as is. If you need to normalize the result of input_value , the best option is to call html_escape on it and compare the resulting string.","ref":"Phoenix.HTML.Form.html#input_value/2","title":"Phoenix.HTML.Form.input_value/2","type":"function"},{"doc":"Same as inputs_for(form, field, []) .","ref":"Phoenix.HTML.Form.html#inputs_for/2","title":"Phoenix.HTML.Form.inputs_for/2","type":"function"},{"doc":"Generate a new form builder for the given parameter in form without an anonymous function. This functionality exists mostly for integration with Phoenix.LiveView that replaces the anonymous function for returning the generated forms instead. Keep in mind that this function does not generate hidden inputs automatically like inputs_for/4 . To generate them you need to explicit do it by yourself. &lt; % = f = form_for @changeset , Routes . user_path ( @conn , :create ) , opts % &gt; Name : &lt; % = text_input f , :name % &gt; &lt; % = for friend_form &lt;- inputs_for ( f , :friends ) do % &gt; # for generating hidden inputs. &lt; % = hidden_inputs_for ( friend_form ) % &gt; &lt; % = text_input friend_form , :name % &gt; &lt; % end % &gt; &lt; / form &gt; See inputs_for/4 for the available options.","ref":"Phoenix.HTML.Form.html#inputs_for/3","title":"Phoenix.HTML.Form.inputs_for/3","type":"function"},{"doc":"Generate a new form builder for the given parameter in form. See the module documentation for examples of using this function. Options :id - the id to be used in the form, defaults to the concatenation of the given field to the parent form id :as - the name to be used in the form, defaults to the concatenation of the given field to the parent form name :default - the value to use if none is available :prepend - the values to prepend when rendering. This only applies if the field value is a list and no parameters were sent through the form. :append - the values to append when rendering. This only applies if the field value is a list and no parameters were sent through the form. :skip_hidden - skip the automatic rendering of hidden fields to allow for more tight control over the generated markup. You can access form.hidden to generate them manually within the supplied callback.","ref":"Phoenix.HTML.Form.html#inputs_for/4","title":"Phoenix.HTML.Form.inputs_for/4","type":"function"},{"doc":"Generates a label tag. Useful when wrapping another input inside a label. Examples label do radio_button :user , :choice , &quot;Choice&quot; end #=&gt; &lt;label class=&quot;control-label&quot;&gt;...&lt;/label&gt; label class : &quot;control-label&quot; do radio_button :user , :choice , &quot;Choice&quot; end #=&gt; &lt;label class=&quot;control-label&quot;&gt;...&lt;/label&gt;","ref":"Phoenix.HTML.Form.html#label/1","title":"Phoenix.HTML.Form.label/1","type":"function"},{"doc":"Generates a label tag for the given field. The form should either be a Phoenix.HTML.Form emitted by form_for or an atom. All given options are forwarded to the underlying tag. A default value is provided for for attribute but can be overriden if you pass a value to the for option. Text content would be inferred from field if not specified as either a function argument or string value in a block. To wrap a label around an input, see label/1 . Examples # Assuming form contains a User schema label ( form , :name , &quot;Name&quot; ) #=&gt; &lt;label for=&quot;user_name&quot;&gt;Name&lt;/label&gt; label ( :user , :email , &quot;Email&quot; ) #=&gt; &lt;label for=&quot;user_email&quot;&gt;Email&lt;/label&gt; label ( :user , :email ) #=&gt; &lt;label for=&quot;user_email&quot;&gt;Email&lt;/label&gt; label ( :user , :email , class : &quot;control-label&quot; ) #=&gt; &lt;label for=&quot;user_email&quot; class=&quot;control-label&quot;&gt;Email&lt;/label&gt; label :user , :email do &quot;E-mail Address&quot; end #=&gt; &lt;label for=&quot;user_email&quot;&gt;E-mail Address&lt;/label&gt; label :user , :email , &quot;E-mail Address&quot; , class : &quot;control-label&quot; #=&gt; &lt;label class=&quot;control-label&quot; for=&quot;user_email&quot;&gt;E-mail Address&lt;/label&gt; label :user , :email , class : &quot;control-label&quot; do &quot;E-mail Address&quot; end #=&gt; &lt;label class=&quot;control-label&quot; for=&quot;user_email&quot;&gt;E-mail Address&lt;/label&gt;","ref":"Phoenix.HTML.Form.html#label/2","title":"Phoenix.HTML.Form.label/2","type":"function"},{"doc":"See label/2 .","ref":"Phoenix.HTML.Form.html#label/3","title":"Phoenix.HTML.Form.label/3","type":"function"},{"doc":"See label/2 .","ref":"Phoenix.HTML.Form.html#label/4","title":"Phoenix.HTML.Form.label/4","type":"function"},{"doc":"Generates a select tag with the given options . Values are expected to be an Enumerable containing two-item tuples (like maps and keyword lists) or any Enumerable where the element will be used both as key and value for the generated select. Examples # Assuming form contains a User schema multiple_select ( form , :roles , [ &quot;Admin&quot; : 1 , &quot;Power User&quot; : 2 ] ) #=&gt; &lt;select id=&quot;user_roles&quot; name=&quot;user[roles][]&quot;&gt; #=&gt; &lt;option value=&quot;1&quot;&gt;Admin&lt;/option&gt; #=&gt; &lt;option value=&quot;2&quot;&gt;Power User&lt;/option&gt; #=&gt; &lt;/select&gt; multiple_select ( form , :roles , [ &quot;Admin&quot; : 1 , &quot;Power User&quot; : 2 ] , selected : [ 1 ] ) #=&gt; &lt;select id=&quot;user_roles&quot; name=&quot;user[roles][]&quot;&gt; #=&gt; &lt;option value=&quot;1&quot; selected=&quot;selected&quot;&gt;Admin&lt;/option&gt; #=&gt; &lt;option value=&quot;2&quot;&gt;Power User&lt;/option&gt; #=&gt; &lt;/select&gt; When working with structs, associations and embeds, you will need to tell Phoenix how to extract the value out of the collection. For example, imagine user.roles is a list of %Role{} structs. You must call it as: multiple_select ( form , :roles , [ &quot;Admin&quot; : 1 , &quot;Power User&quot; : 2 ] , selected : Enum . map ( @user . roles , &amp; ( &amp;1 . id ) ) The :selected option will mark the given IDs as selected unless the form is being resubmitted. When resubmitted, it uses the form params as values. Options :selected - the default options to be marked as selected. The values on this list are ignored in case ids have been set as parameters. All other options are forwarded to the underlying HTML tag.","ref":"Phoenix.HTML.Form.html#multiple_select/4","title":"Phoenix.HTML.Form.multiple_select/4","type":"function"},{"doc":"Generates a number input. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#number_input/3","title":"Phoenix.HTML.Form.number_input/3","type":"function"},{"doc":"Returns options to be used inside a select. This is useful when building the select by hand. It expects all options and one or more select values. Examples options_for_select ( [ &quot;Admin&quot; : &quot;admin&quot; , &quot;User&quot; : &quot;user&quot; ] , &quot;admin&quot; ) #=&gt; &lt;option value=&quot;admin&quot; selected=&quot;selected&quot;&gt;Admin&lt;/option&gt; #=&gt; &lt;option value=&quot;user&quot;&gt;User&lt;/option&gt; Groups are also supported: options_for_select ( [ &quot;Europe&quot; : [ &quot;UK&quot; , &quot;Sweden&quot; , &quot;France&quot; ] , ... ] , nil ) #=&gt; &lt;optgroup label=&quot;Europe&quot;&gt; #=&gt; &lt;option&gt;UK&lt;/option&gt; #=&gt; &lt;option&gt;Sweden&lt;/option&gt; #=&gt; &lt;option&gt;France&lt;/option&gt; #=&gt; &lt;/optgroup&gt;","ref":"Phoenix.HTML.Form.html#options_for_select/2","title":"Phoenix.HTML.Form.options_for_select/2","type":"function"},{"doc":"Generates a password input. For security reasons, the form data and parameter values are never re-used in password_input/3 . Pass the value explicitly if you would like to set one. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#password_input/3","title":"Phoenix.HTML.Form.password_input/3","type":"function"},{"doc":"Generates a radio button. Invoke this function for each possible value you want to be sent to the server. Examples # Assuming form contains a User schema radio_button ( form , :role , &quot;admin&quot; ) #=&gt; &lt;input id=&quot;user_role_admin&quot; name=&quot;user[role]&quot; type=&quot;radio&quot; value=&quot;admin&quot;&gt; Options All options are simply forwarded to the underlying HTML tag.","ref":"Phoenix.HTML.Form.html#radio_button/4","title":"Phoenix.HTML.Form.radio_button/4","type":"function"},{"doc":"Generates a range input. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#range_input/3","title":"Phoenix.HTML.Form.range_input/3","type":"function"},{"doc":"Generates a reset input to reset all the form fields to their original state. All options are forwarded to the underlying input tag. Examples reset &quot;Reset&quot; #=&gt; &lt;input type=&quot;reset&quot; value=&quot;Reset&quot;&gt; reset &quot;Reset&quot; , class : &quot;btn&quot; #=&gt; &lt;input type=&quot;reset&quot; value=&quot;Reset&quot; class=&quot;btn&quot;&gt;","ref":"Phoenix.HTML.Form.html#reset/2","title":"Phoenix.HTML.Form.reset/2","type":"function"},{"doc":"Generates a search input. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#search_input/3","title":"Phoenix.HTML.Form.search_input/3","type":"function"},{"doc":"Generates a select tag with the given options . options are expected to be an enumerable which will be used to generate each respective option . The enumerable may have: keyword lists - each keyword list is expected to have the keys :key and :value . Additional keys such as :disabled may be given to customize the option two-item tuples - where the first element is an atom, string or integer to be used as the option label and the second element is an atom, string or integer to be used as the option value atom, string or integer - which will be used as both label and value for the generated select Optgroups If options is map or keyword list where the first element is a string, atom or integer and the second element is a list or a map, it is assumed the key will be wrapped in an &lt;optgroup&gt; and the value will be used to generate &lt;options&gt; nested under the group. Examples # Assuming form contains a User schema select ( form , :age , 0 .. 120 ) #=&gt; &lt;select id=&quot;user_age&quot; name=&quot;user[age]&quot;&gt; #=&gt; &lt;option value=&quot;0&quot;&gt;0&lt;/option&gt; #=&gt; ... #=&gt; &lt;option value=&quot;120&quot;&gt;120&lt;/option&gt; #=&gt; &lt;/select&gt; select ( form , :role , [ &quot;Admin&quot; : &quot;admin&quot; , &quot;User&quot; : &quot;user&quot; ] ) #=&gt; &lt;select id=&quot;user_role&quot; name=&quot;user[role]&quot;&gt; #=&gt; &lt;option value=&quot;admin&quot;&gt;Admin&lt;/option&gt; #=&gt; &lt;option value=&quot;user&quot;&gt;User&lt;/option&gt; #=&gt; &lt;/select&gt; select ( form , :role , [ [ key : &quot;Admin&quot; , value : &quot;admin&quot; , disabled : true ] , [ key : &quot;User&quot; , value : &quot;user&quot; ] ] ) #=&gt; &lt;select id=&quot;user_role&quot; name=&quot;user[role]&quot;&gt; #=&gt; &lt;option value=&quot;admin&quot; disabled=&quot;disabled&quot;&gt;Admin&lt;/option&gt; #=&gt; &lt;option value=&quot;user&quot;&gt;User&lt;/option&gt; #=&gt; &lt;/select&gt; You can also pass a prompt: select ( form , :role , [ &quot;Admin&quot; : &quot;admin&quot; , &quot;User&quot; : &quot;user&quot; ] , prompt : &quot;Choose your role&quot; ) #=&gt; &lt;select id=&quot;user_role&quot; name=&quot;user[role]&quot;&gt; #=&gt; &lt;option value=&quot;&quot;&gt;Choose your role&lt;/option&gt; #=&gt; &lt;option value=&quot;admin&quot;&gt;Admin&lt;/option&gt; #=&gt; &lt;option value=&quot;user&quot;&gt;User&lt;/option&gt; #=&gt; &lt;/select&gt; And customize the prompt as any other entry: select ( form , :role , [ &quot;Admin&quot; : &quot;admin&quot; , &quot;User&quot; : &quot;user&quot; ] , prompt : [ key : &quot;Choose your role&quot; , disabled : true ] ) #=&gt; &lt;select id=&quot;user_role&quot; name=&quot;user[role]&quot;&gt; #=&gt; &lt;option value=&quot;&quot; disabled=&quot;&quot;&gt;Choose your role&lt;/option&gt; #=&gt; &lt;option value=&quot;admin&quot;&gt;Admin&lt;/option&gt; #=&gt; &lt;option value=&quot;user&quot;&gt;User&lt;/option&gt; #=&gt; &lt;/select&gt; If you want to select an option that comes from the database, such as a manager for a given project, you may write: select ( form , :manager_id , Enum . map ( @managers , &amp; { &amp;1 . name , &amp;1 . id } ) ) #=&gt; &lt;select id=&quot;manager_id&quot; name=&quot;project[manager_id]&quot;&gt; #=&gt; &lt;option value=&quot;1&quot;&gt;Mary Jane&lt;/option&gt; #=&gt; &lt;option value=&quot;2&quot;&gt;John Doe&lt;/option&gt; #=&gt; &lt;/select&gt; Finally, if the values are a list or a map, we use the keys for grouping: select ( form , :country , [ &quot;Europe&quot; : [ &quot;UK&quot; , &quot;Sweden&quot; , &quot;France&quot; ] ] , ... ) #=&gt; &lt;select id=&quot;user_country&quot; name=&quot;user[country]&quot;&gt; #=&gt; &lt;optgroup label=&quot;Europe&quot;&gt; #=&gt; &lt;option&gt;UK&lt;/option&gt; #=&gt; &lt;option&gt;Sweden&lt;/option&gt; #=&gt; &lt;option&gt;France&lt;/option&gt; #=&gt; &lt;/optgroup&gt; #=&gt; ... #=&gt; &lt;/select&gt; Options :prompt - an option to include at the top of the options. It may be a string or a keyword list of atttributes and the :key :selected - the default value to use when none was sent as parameter Be aware that a :multiple option will not generate a correctly functioning multiple select element. Use multiple_select/4 instead. All other options are forwarded to the underlying HTML tag.","ref":"Phoenix.HTML.Form.html#select/4","title":"Phoenix.HTML.Form.select/4","type":"function"},{"doc":"Generates a submit button to send the form. Examples submit do : &quot;Submit&quot; #=&gt; &lt;button type=&quot;submit&quot;&gt;Submit&lt;/button&gt;","ref":"Phoenix.HTML.Form.html#submit/1","title":"Phoenix.HTML.Form.submit/1","type":"function"},{"doc":"Generates a submit button to send the form. All options are forwarded to the underlying button tag. When called with a do: block, the button tag options come first. Examples submit &quot;Submit&quot; #=&gt; &lt;button type=&quot;submit&quot;&gt;Submit&lt;/button&gt; submit &quot;Submit&quot; , class : &quot;btn&quot; #=&gt; &lt;button class=&quot;btn&quot; type=&quot;submit&quot;&gt;Submit&lt;/button&gt; submit [ class : &quot;btn&quot; ] , do : &quot;Submit&quot; #=&gt; &lt;button class=&quot;btn&quot; type=&quot;submit&quot;&gt;Submit&lt;/button&gt;","ref":"Phoenix.HTML.Form.html#submit/2","title":"Phoenix.HTML.Form.submit/2","type":"function"},{"doc":"Generates a telephone input. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#telephone_input/3","title":"Phoenix.HTML.Form.telephone_input/3","type":"function"},{"doc":"Generates a text input. The form should either be a Phoenix.HTML.Form emitted by form_for or an atom. All given options are forwarded to the underlying input, default values are provided for id, name and value if possible. Examples # Assuming form contains a User schema text_input ( form , :name ) #=&gt; &lt;input id=&quot;user_name&quot; name=&quot;user[name]&quot; type=&quot;text&quot; value=&quot;&quot;&gt; text_input ( :user , :name ) #=&gt; &lt;input id=&quot;user_name&quot; name=&quot;user[name]&quot; type=&quot;text&quot; value=&quot;&quot;&gt;","ref":"Phoenix.HTML.Form.html#text_input/3","title":"Phoenix.HTML.Form.text_input/3","type":"function"},{"doc":"Generates a textarea input. All given options are forwarded to the underlying input, default values are provided for id, name and textarea content if possible. Examples # Assuming form contains a User schema textarea ( form , :description ) #=&gt; &lt;textarea id=&quot;user_description&quot; name=&quot;user[description]&quot;&gt;&lt;/textarea&gt; New lines Notice the generated textarea includes a new line after the opening tag. This is because the HTML spec says new lines after tags must be ignored and all major browser implementations do that. So in order to avoid new lines provided by the user from being ignored when the form is resubmitted, we automatically add a new line before the text area value.","ref":"Phoenix.HTML.Form.html#textarea/3","title":"Phoenix.HTML.Form.textarea/3","type":"function"},{"doc":"Generates a time input. Warning: this feature isn't available in all browsers. Check http://caniuse.com/#feat=input-datetime for further informations. Options :precision - Allowed values: :minute , :second , :millisecond . Defaults to :minute . All other options are forwarded. See text_input/3 for example and docs. Examples time_input form , :time #=&gt; &lt;input id=&quot;form_time&quot; name=&quot;form[time]&quot; type=&quot;time&quot; value=&quot;23:00&quot;&gt; time_input form , :time , precision : :second #=&gt; &lt;input id=&quot;form_time&quot; name=&quot;form[time]&quot; type=&quot;time&quot; value=&quot;23:00:00&quot;&gt; time_input form , :time , precision : :millisecond #=&gt; &lt;input id=&quot;form_time&quot; name=&quot;form[time]&quot; type=&quot;time&quot; value=&quot;23:00:00.000&quot;&gt;","ref":"Phoenix.HTML.Form.html#time_input/3","title":"Phoenix.HTML.Form.time_input/3","type":"function"},{"doc":"Generates select tags for time. Check datetime_select/3 for more information on options and supported values.","ref":"Phoenix.HTML.Form.html#time_select/3","title":"Phoenix.HTML.Form.time_select/3","type":"function"},{"doc":"Generates an url input. See text_input/3 for example and docs.","ref":"Phoenix.HTML.Form.html#url_input/3","title":"Phoenix.HTML.Form.url_input/3","type":"function"},{"doc":"","ref":"Phoenix.HTML.Form.html#t:field/0","title":"Phoenix.HTML.Form.field/0","type":"type"},{"doc":"","ref":"Phoenix.HTML.Form.html#t:t/0","title":"Phoenix.HTML.Form.t/0","type":"type"},{"doc":"Converts a data structure into a Phoenix.HTML.Form struct.","ref":"Phoenix.HTML.FormData.html","title":"Phoenix.HTML.FormData","type":"protocol"},{"doc":"Receives the given field and returns its input type (:text_input, :select, etc). Returns nil if the type is unknown.","ref":"Phoenix.HTML.FormData.html#input_type/3","title":"Phoenix.HTML.FormData.input_type/3","type":"function"},{"doc":"Returns the HTML5 validations that would apply to the given field.","ref":"Phoenix.HTML.FormData.html#input_validations/3","title":"Phoenix.HTML.FormData.input_validations/3","type":"function"},{"doc":"Returns the value for the given field.","ref":"Phoenix.HTML.FormData.html#input_value/3","title":"Phoenix.HTML.FormData.input_value/3","type":"function"},{"doc":"Converts a data structure into a Phoenix.HTML.Form struct. The options are the same options given to form_for/4 . It can be used by implementations to configure their behaviour and it must be stored in the underlying struct, with any custom field removed.","ref":"Phoenix.HTML.FormData.html#to_form/2","title":"Phoenix.HTML.FormData.to_form/2","type":"function"},{"doc":"Converts the field in the given form based on the data structure into a list of Phoenix.HTML.Form structs. The options are the same options given to inputs_for/4 . It can be used by implementations to configure their behaviour and it must be stored in the underlying struct, with any custom field removed.","ref":"Phoenix.HTML.FormData.html#to_form/4","title":"Phoenix.HTML.FormData.to_form/4","type":"function"},{"doc":"","ref":"Phoenix.HTML.FormData.html#t:t/0","title":"Phoenix.HTML.FormData.t/0","type":"type"},{"doc":"Helpers related to formatting text.","ref":"Phoenix.HTML.Format.html","title":"Phoenix.HTML.Format","type":"module"},{"doc":"Returns text transformed into HTML using simple formatting rules. Two or more consecutive newlines \\n\\n or \\r\\n\\r\\n are considered as a paragraph and text between them is wrapped in &lt;p&gt; tags. One newline \\n or \\r\\n is considered as a linebreak and a &lt;br&gt; tag is inserted. Examples iex&gt; text_to_html ( &quot;Hello \\n \\n World&quot; ) |&gt; safe_to_string &quot;&lt;p&gt;Hello&lt;/p&gt; \\n &lt;p&gt;World&lt;/p&gt; \\n &quot; iex&gt; text_to_html ( &quot;Hello \\n World&quot; ) |&gt; safe_to_string &quot;&lt;p&gt;Hello&lt;br&gt; \\n World&lt;/p&gt; \\n &quot; iex&gt; opts = [ wrapper_tag : :div , attributes : [ class : &quot;p&quot; ] ] ...&gt; text_to_html ( &quot;Hello \\n \\n World&quot; , opts ) |&gt; safe_to_string &quot;&lt;div class= \\&quot; p \\&quot; &gt;Hello&lt;/div&gt; \\n &lt;div class= \\&quot; p \\&quot; &gt;World&lt;/div&gt; \\n &quot; Options :escape - if false does not html escape input (default: true ) :wrapper_tag - tag to wrap each paragraph (default: :p ) :attributes - html attributes of the wrapper tag (default: [] ) :insert_brs - if true insert &lt;br&gt; for single line breaks (default: true )","ref":"Phoenix.HTML.Format.html#text_to_html/2","title":"Phoenix.HTML.Format.text_to_html/2","type":"function"},{"doc":"Conveniences for working with links and URLs in HTML.","ref":"Phoenix.HTML.Link.html","title":"Phoenix.HTML.Link","type":"module"},{"doc":"Generates a button tag that uses the Javascript function handleClick() (see phoenix_html.js) to submit the form data. Useful to ensure that links that change data are not triggered by search engines and other spidering software. Examples button ( &quot;hello&quot; , to : &quot;/world&quot; ) #=&gt; &lt;button class=&quot;button&quot; data-csrf=&quot;csrf_token&quot; data-method=&quot;post&quot; data-to=&quot;/world&quot;&gt;hello&lt;/button&gt; button ( &quot;hello&quot; , to : &quot;/world&quot; , method : :get , class : &quot;btn&quot; ) #=&gt; &lt;button class=&quot;btn&quot; data-method=&quot;get&quot; data-to=&quot;/world&quot;&gt;hello&lt;/button&gt; Options :to - the page to link to. This option is required :method - the method to use with the button. Defaults to :post. All other options are forwarded to the underlying button input. When the :method is set to :get and the :to URL contains query parameters the generated form element will strip the parameters in accordance with the W3C form specification. Data attributes Data attributes are added as a keyword list passed to the data key. The following data attributes are supported: data-confirm - shows a confirmation prompt before generating and submitting the form.","ref":"Phoenix.HTML.Link.html#button/2","title":"Phoenix.HTML.Link.button/2","type":"function"},{"doc":"Generates a link to the given URL. Examples link ( &quot;hello&quot; , to : &quot;/world&quot; ) #=&gt; &lt;a href=&quot;/world&quot;&gt;hello&lt;/a&gt; link ( &quot;hello&quot; , to : URI . parse ( &quot;https://elixir-lang.org&quot; ) ) #=&gt; &lt;a href=&quot;https://elixir-lang.org&quot;&gt;hello&lt;/a&gt; link ( &quot;&lt;hello&gt;&quot; , to : &quot;/world&quot; ) #=&gt; &lt;a href=&quot;/world&quot;&gt;&amp;lt;hello&amp;gt;&lt;/a&gt; link ( &quot;&lt;hello&gt;&quot; , to : &quot;/world&quot; , class : &quot;btn&quot; ) #=&gt; &lt;a class=&quot;btn&quot; href=&quot;/world&quot;&gt;&amp;lt;hello&amp;gt;&lt;/a&gt; link ( &quot;delete&quot; , to : &quot;/the_world&quot; , data : [ confirm : &quot;Really?&quot; ] ) #=&gt; &lt;a data-confirm=&quot;Really?&quot; href=&quot;/the_world&quot;&gt;delete&lt;/a&gt; # If you supply a method other than `:get`: link ( &quot;delete&quot; , to : &quot;/everything&quot; , method : :delete ) #=&gt; &lt;a href=&quot;/everything&quot; data-csrf=&quot;csrf_token&quot; data-method=&quot;delete&quot; data-to=&quot;/everything&quot;&gt;delete&lt;/a&gt; # You can use a `do ... end` block too: link to : &quot;/hello&quot; do &quot;world&quot; end Options :to - the page to link to. This option is required :method - the method to use with the link. In case the method is not :get , the link is generated inside the form which sets the proper information. In order to submit the form, JavaScript must be enabled :csrf_token - a custom token to use for links with a method other than :get . All other options are forwarded to the underlying &lt;a&gt; tag. JavaScript dependency In order to support links where :method is not :get or use the above data attributes, Phoenix.HTML relies on JavaScript. You can load priv/static/phoenix_html.js into your build tool. Data attributes Data attributes are added as a keyword list passed to the data key. The following data attributes are supported: data-confirm - shows a confirmation prompt before generating and submitting the form when :method is not :get . Overriding the default confirm behaviour phoenix_html.js does trigger a custom event phoenix.link.click on the clicked DOM element when a click happened. This allows you to intercept the event on it's way bubbling up to window and do your own custom logic to enhance or replace how the data-confirm attribute is handled. You could for example replace the browsers confirm() behavior with a custom javascript implementation: // listen on document.body, so it's executed before the default of // phoenix_html, which is listening on the window object document.body.addEventListener('phoenix.link.click', function (e) { // Prevent default implementation e.stopPropagation(); // Introduce alternative implementation var message = e.target.getAttribute(&quot;data-confirm&quot;); if(!message){ return true; } vex.dialog.confirm({ message: message, callback: function (value) { if (value == false) { e.preventDefault(); } } }) }, false); Or you could attach your own custom behavior. window.addEventListener('phoenix.link.click', function (e) { // Introduce custom behaviour var message = e.target.getAttribute(&quot;data-prompt&quot;); var answer = e.target.getAttribute(&quot;data-prompt-answer&quot;); if(message &amp;&amp; answer &amp;&amp; (answer != window.prompt(message))) { e.preventDefault(); } }, false); The latter could also be bound to any click event, but this way you can be sure your custom code is only executed when the code of phoenix_html.js is run. CSRF Protection By default, CSRF tokens are generated through Plug.CSRFProtection .","ref":"Phoenix.HTML.Link.html#link/2","title":"Phoenix.HTML.Link.link/2","type":"function"},{"doc":"Defines the HTML safe protocol. In order to promote HTML safety, Phoenix templates do not use Kernel.to_string/1 to convert data types to strings in templates. Instead, Phoenix uses this protocol which must be implemented by data structures and guarantee that a HTML safe representation is returned. Furthermore, this protocol relies on iodata, which provides better performance when sending or streaming data to the client.","ref":"Phoenix.HTML.Safe.html","title":"Phoenix.HTML.Safe","type":"protocol"},{"doc":"","ref":"Phoenix.HTML.Safe.html#to_iodata/1","title":"Phoenix.HTML.Safe.to_iodata/1","type":"function"},{"doc":"","ref":"Phoenix.HTML.Safe.html#t:t/0","title":"Phoenix.HTML.Safe.t/0","type":"type"},{"doc":"Helpers related to producing HTML tags within templates. Note the examples in this module use safe_to_string/1 imported from Phoenix.HTML for readability.","ref":"Phoenix.HTML.Tag.html","title":"Phoenix.HTML.Tag","type":"module"},{"doc":"Escapes a list of attributes, returning iodata. Pay attention that, unlike tag/2 and content_tag/2 , this function does not sort the attributes. iex&gt; attributes_escape ( title : &quot;the title&quot; , id : &quot;the id&quot; , selected : true ) { :safe , [ [ 32 , &quot;title&quot; , 61 , 34 , &quot;the title&quot; , 34 ] , [ 32 , &quot;id&quot; , 61 , 34 , &quot;the id&quot; , 34 ] , [ 32 , &quot;selected&quot; ] ] }","ref":"Phoenix.HTML.Tag.html#attributes_escape/1","title":"Phoenix.HTML.Tag.attributes_escape/1","type":"function"},{"doc":"Creates an HTML tag with given name, content, and attributes. See Phoenix.HTML.Tag.tag/2 for more information and examples. iex&gt; safe_to_string content_tag ( :p , &quot;Hello&quot; ) &quot;&lt;p&gt;Hello&lt;/p&gt;&quot; iex&gt; safe_to_string content_tag ( :p , &quot;&lt;Hello&gt;&quot; , class : &quot;test&quot; ) &quot;&lt;p class= \\&quot; test \\&quot; &gt;&amp;lt;Hello&amp;gt;&lt;/p&gt;&quot; iex&gt; safe_to_string ( content_tag :p , class : &quot;test&quot; do ...&gt; &quot;Hello&quot; ...&gt; end ) &quot;&lt;p class= \\&quot; test \\&quot; &gt;Hello&lt;/p&gt;&quot; iex&gt; safe_to_string content_tag ( :option , &quot;Display Value&quot; , [ { :data , [ foo : &quot;bar&quot; ] } , value : &quot;value&quot; ] ) &quot;&lt;option data-foo= \\&quot; bar \\&quot; value= \\&quot; value \\&quot; &gt;Display Value&lt;/option&gt;&quot;","ref":"Phoenix.HTML.Tag.html#content_tag/2","title":"Phoenix.HTML.Tag.content_tag/2","type":"function"},{"doc":"","ref":"Phoenix.HTML.Tag.html#content_tag/3","title":"Phoenix.HTML.Tag.content_tag/3","type":"function"},{"doc":"Generates a meta tag with CSRF information. Additional options to the tag can be given.","ref":"Phoenix.HTML.Tag.html#csrf_meta_tag/1","title":"Phoenix.HTML.Tag.csrf_meta_tag/1","type":"function"},{"doc":"Returns the csrf_token value to be used by forms, meta tags, etc. By default, CSRF tokens are generated through Plug.CSRFProtection which is capable of generating a separate token per host. Therefore it is recommended to pass the URI of the destination as argument. If none is given %URI{host: nil} is used, which implies a local request is being done.","ref":"Phoenix.HTML.Tag.html#csrf_token_value/1","title":"Phoenix.HTML.Tag.csrf_token_value/1","type":"function"},{"doc":"Generates a form tag. This function generates the &lt;form&gt; tag without its closing part. Check form_tag/3 for generating an enclosing tag. Examples form_tag ( &quot;/hello&quot; ) &lt; form action = &quot;/hello&quot; method = &quot;post&quot; &gt; form_tag ( &quot;/hello&quot; , method : :get ) &lt; form action = &quot;/hello&quot; method = &quot;get&quot; &gt; Options :method - the HTTP method. If the method is not &quot;get&quot; nor &quot;post&quot;, an input tag with name _method is generated along-side the form tag. Defaults to &quot;post&quot;. :multipart - when true, sets enctype to &quot;multipart/form-data&quot;. Required when uploading files :csrf_token - for &quot;post&quot; requests, the form tag will automatically include an input tag with name _csrf_token . When set to false, this is disabled All other options are passed to the underlying HTML tag. CSRF Protection By default, CSRF tokens are generated through Plug.CSRFProtection .","ref":"Phoenix.HTML.Tag.html#form_tag/2","title":"Phoenix.HTML.Tag.form_tag/2","type":"function"},{"doc":"Generates a form tag with the given contents. Examples form_tag ( &quot;/hello&quot; , method : &quot;get&quot; ) do &quot;Hello&quot; end &lt; form action = &quot;/hello&quot; method = &quot;get&quot; &gt; ... Hello ... &lt; / form &gt;","ref":"Phoenix.HTML.Tag.html#form_tag/3","title":"Phoenix.HTML.Tag.form_tag/3","type":"function"},{"doc":"Generates an img tag with a src. Examples img_tag ( user . photo_path ) &lt; img src = &quot;/photo.png&quot; &gt; img_tag ( user . photo , class : &quot;image&quot; ) &lt; img src = &quot;/smile.png&quot; class = &quot;image&quot; &gt; To generate a path to an image hosted in your application &quot;priv/static&quot;, with the @conn endpoint, use static_path/2 to get a URL with cache control parameters: img_tag ( Routes . static_path ( @conn , &quot;/logo.png&quot; ) ) &lt; img src = &quot;/logo-123456.png?vsn=d&quot; &gt; For responsive images, pass a map, list or string through :srcset . img_tag ( &quot;/logo.png&quot; , srcset : %{ &quot;/logo.png&quot; =&gt; &quot;1x&quot; , &quot;/logo-2x.png&quot; =&gt; &quot;2x&quot; } ) &lt; img src = &quot;/logo.png&quot; srcset = &quot;/logo.png 1x, /logo-2x.png 2x&quot; &gt; img_tag ( &quot;/logo.png&quot; , srcset : [ &quot;/logo.png&quot; , { &quot;/logo-2x.png&quot; , &quot;2x&quot; } ] ) &lt; img src = &quot;/logo.png&quot; srcset = &quot;/logo.png, /logo-2x.png 2x&quot; &gt;","ref":"Phoenix.HTML.Tag.html#img_tag/2","title":"Phoenix.HTML.Tag.img_tag/2","type":"function"},{"doc":"Creates an HTML tag with the given name and options. iex&gt; safe_to_string tag ( :br ) &quot;&lt;br&gt;&quot; iex&gt; safe_to_string tag ( :input , type : &quot;text&quot; , name : &quot;user_id&quot; ) &quot;&lt;input name= \\&quot; user_id \\&quot; type= \\&quot; text \\&quot; &gt;&quot; Data attributes In order to add custom data attributes you need to pass a tuple containing :data atom and a keyword list with data attributes' names and values as the first element in the tag's attributes keyword list: iex&gt; safe_to_string tag ( :input , [ data : [ foo : &quot;bar&quot; ] , id : &quot;some_id&quot; ] ) &quot;&lt;input data-foo= \\&quot; bar \\&quot; id= \\&quot; some_id \\&quot; &gt;&quot; Boolean values In case an attribute contains a boolean value, its key is repeated when it is true, as expected in HTML, or the attribute is completely removed if it is false: iex&gt; safe_to_string tag ( :audio , autoplay : &quot;autoplay&quot; ) &quot;&lt;audio autoplay= \\&quot; autoplay \\&quot; &gt;&quot; iex&gt; safe_to_string tag ( :audio , autoplay : true ) &quot;&lt;audio autoplay&gt;&quot; iex&gt; safe_to_string tag ( :audio , autoplay : false ) &quot;&lt;audio&gt;&quot; If you want the boolean attribute to be sent as is, you can explicitly convert it to a string before.","ref":"Phoenix.HTML.Tag.html#tag/1","title":"Phoenix.HTML.Tag.tag/1","type":"function"},{"doc":"","ref":"Phoenix.HTML.Tag.html#tag/2","title":"Phoenix.HTML.Tag.tag/2","type":"function"}]